#!/bin/bash
#===============================================================================
#   weight.sh       |   Version 0.8 |   FreeBSD License |   2013-03-28
#   James Hendrie   |   hendrie dot james at gmail dot com
#===============================================================================

##  Determine where our weight.log file is
if [ ! -e "$HOME/.weight/weight.log" ]; then
    if [ ! -d "$HOME/.weight" ]; then
        mkdir "$HOME/.weight"
    fi

    if [ ! -d "$HOME/.weight" ]; then
        echo "Error:  Cannot create directory $USER/.weight" 2>&1
    fi

    touch "$HOME/.weight/weight.log"
fi


##  Double-check, finally set the weight log variable
if [ ! -e "$HOME/.weight/weight.log" ]; then
    echo "Error:  $HOME/.weight/weight.log does not exist." 2>&1
    if [ ! -e "weight.log" ]; then
        echo "Attempting to create file in present working directory:" 2>&1
        echo "($PWD/weight.log)" 2>&1
        touch "weight.log"
    fi

    weightLog="weight.log"

else
    weightLog="$HOME/.weight/weight.log"
fi


##  Set the current date, time and username.  That last part is a bit redundant,
##  but I'll leave it as is in case I feel like further altering this script to
##  be more advanced later.
currentDate=$(date "+%Y-%m-%d")
currentTime=$(date "+%H:%M:%S")
userName="$USER"

##  Set the verbosity
verbosity="0"

##  List the ten most recent entries for $USER, sorted by date/time (by default)
function list_recent
{
    cat $weightLog | grep "$1" | sort | tail
}

##  List ALL entires for the given username, sorted by date/time (by default)
function list_all
{
    cat $weightLog | grep "$1" | sort
}

##  List the ten last entries appended to the weight log, regardless of time or
##  user
function list_tail
{
    tail $weightLog
}

##  Append the given weight to the weight logfile
function insert_weight
{
    ##  Accepted arguments:
    ##  1:  Verbosity option (verbosity)
    ##  2:  User name (userName)
    ##  3:  Weight (weight OR lastArg)
    if [ "$1" = "1" ]; then
        echo "Logging weight as $3 for user $2 on $currentDate at $currentTime"
    fi

    echo -e "$currentDate\t$currentTime\t$2\t$3" >> $weightLog
}


##  Print the help
function print_help
{
    echo -e "Usage:\n"
    echo "  weight [NUMBER]"
    echo "    Log weight"
    echo ""
    echo "  weight [USERNAME]"
    echo "    List ten recent entries for USERNAME"
    echo ""
    echo "  weight"
    echo "    List ten recent entries for yourself"
    echo ""
    echo "  weight [OPTION]"
    echo ""
    echo -e "  weight [OPTION] [USER]\n"
    echo "Valid options are:"
    echo "  -h      Print this help text"
    echo "  -v      Enable verbosity"
    echo "  -l      List all entries for yourself or supplied username"
    echo "  -r      List ten recent entries for yourself or supplied username"
    echo "  -t      List ten last entries appended to the log file (tail)"

}


##  If no arguments are supplied, list the ten most recent entries for $USER
if [ $# -eq 0 ]; then
    list_recent $userName
    exit
fi

##  This determines what the last argument is (most portable)
##  (bash/ksh only:  ${@: -1})
##  (bash 3.x+ only:    ${!#})
for i in $@; do :; done
lastArg="$i"


##  Get options
while getopts "hvlrts" flag
do
    if [ $flag == 'h' ]; then
        print_help
        exit

    elif [ $flag == 'v' ]; then
        verbosity="1"

    elif [ $flag == 'l' ]; then
        listAll="1"

    elif [ $flag == 'r' ]; then
        listRecent="1"

    elif [ $flag == 't' ]; then
        listTail="1"

    elif [ $flag == 's' ]; then
        verbosity="0"
    fi

done


##  If there is only one argument provided
if [ $# = 1 ]; then

    if [ "$listRecent" = "1" ]; then
        list_recent $userName
        exit
    fi

    if [ "$listTail" = "1" ]; then
        list_tail
        exit
    fi

    if [ "$listAll" = "1" ]; then
        list_all $userName
        exit
    fi

    if [ "$verbosity" = "1" ]; then
        list_recent $userName
        exit
    fi

    if [ "$(echo $lastArg | tr -d '[:digit:]')" = "" ]; then
        insert_weight $verbosity $userName $lastArg
    elif [ "$(echo $lastArg | tr -d '[:alpha:]' | \
        tr -d '[:digit:]')" = "" ]; then
        list_recent $lastArg
    else
        echo "Error:  Invalid input:  $lastArg" 2>&1
    fi

##  Otherwise, if more than one argument was provided
else


    if [ "$listAll" = "1" ]; then
        list_all $lastArg
        exit
    fi

    if [ "$listRecent" = "1" ]; then
        list_recent $lastArg
        exit
    fi

    if [ "$listTail" = "1" ]; then
        list_tail
        exit
    fi

    if [ "$(echo $lastArg | tr -d '[:alpha:]' | tr -d '[:space:]' | tr -d '[:cntrl:]')" = "" ]; then
        echo "Error:  $lastArg is not a valid weight." 2>&1
    else
        insert_weight $verbosity $userName $lastArg
    fi
fi
